@@ -1,8 +1,8 @@
 # coding: utf-8
 
-VERSION = (1, 16, 10)
-CODENAME = "COPYparty"
-BUILD_DT = (2025, 1, 25)
+VERSION = (1, 19, 2)
+CODENAME = "usernames"
+BUILD_DT = (2025, 8, 17)
 
 S_VERSION = ".".join(map(str, VERSION))
 S_BUILD_DT = "{0:04d}-{1:02d}-{2:02d}".format(*BUILD_DT)
